# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["StudioProjectCreateParams"]


class StudioProjectCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    description: Required[str]
    """The description of the Studio Project"""

    name: Required[str]
    """The name of the Studio Project"""

    studio_api_key: Required[str]
    """Your API key for Studio, can be updated with the PATCH endpoint"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["FineTuningJobEvent"]


class FineTuningJobEvent(BaseModel):
    level: str

    message: str

    timestamp: Optional[float] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tasks import (
    TasksResource,
    AsyncTasksResource,
    TasksResourceWithRawResponse,
    AsyncTasksResourceWithRawResponse,
    TasksResourceWithStreamingResponse,
    AsyncTasksResourceWithStreamingResponse,
)
from .evaluations import (
    EvaluationsResource,
    AsyncEvaluationsResource,
    EvaluationsResourceWithRawResponse,
    AsyncEvaluationsResourceWithRawResponse,
    EvaluationsResourceWithStreamingResponse,
    AsyncEvaluationsResourceWithStreamingResponse,
)
from .test_case_results import (
    TestCaseResultsResource,
    AsyncTestCaseResultsResource,
    TestCaseResultsResourceWithRawResponse,
    AsyncTestCaseResultsResourceWithRawResponse,
    TestCaseResultsResourceWithStreamingResponse,
    AsyncTestCaseResultsResourceWithStreamingResponse,
)
from .evaluation_metrics import (
    EvaluationMetricsResource,
    AsyncEvaluationMetricsResource,
    EvaluationMetricsResourceWithRawResponse,
    AsyncEvaluationMetricsResourceWithRawResponse,
    EvaluationMetricsResourceWithStreamingResponse,
    AsyncEvaluationMetricsResourceWithStreamingResponse,
)
from .contributor_metrics import (
    ContributorMetricsResource,
    AsyncContributorMetricsResource,
    ContributorMetricsResourceWithRawResponse,
    AsyncContributorMetricsResourceWithRawResponse,
    ContributorMetricsResourceWithStreamingResponse,
    AsyncContributorMetricsResourceWithStreamingResponse,
)
from .hybrid_eval_metrics import (
    HybridEvalMetricsResource,
    AsyncHybridEvalMetricsResource,
    HybridEvalMetricsResourceWithRawResponse,
    AsyncHybridEvalMetricsResourceWithRawResponse,
    HybridEvalMetricsResourceWithStreamingResponse,
    AsyncHybridEvalMetricsResourceWithStreamingResponse,
)

__all__ = [
    "TasksResource",
    "AsyncTasksResource",
    "TasksResourceWithRawResponse",
    "AsyncTasksResourceWithRawResponse",
    "TasksResourceWithStreamingResponse",
    "AsyncTasksResourceWithStreamingResponse",
    "ContributorMetricsResource",
    "AsyncContributorMetricsResource",
    "ContributorMetricsResourceWithRawResponse",
    "AsyncContributorMetricsResourceWithRawResponse",
    "ContributorMetricsResourceWithStreamingResponse",
    "AsyncContributorMetricsResourceWithStreamingResponse",
    "EvaluationMetricsResource",
    "AsyncEvaluationMetricsResource",
    "EvaluationMetricsResourceWithRawResponse",
    "AsyncEvaluationMetricsResourceWithRawResponse",
    "EvaluationMetricsResourceWithStreamingResponse",
    "AsyncEvaluationMetricsResourceWithStreamingResponse",
    "HybridEvalMetricsResource",
    "AsyncHybridEvalMetricsResource",
    "HybridEvalMetricsResourceWithRawResponse",
    "AsyncHybridEvalMetricsResourceWithRawResponse",
    "HybridEvalMetricsResourceWithStreamingResponse",
    "AsyncHybridEvalMetricsResourceWithStreamingResponse",
    "TestCaseResultsResource",
    "AsyncTestCaseResultsResource",
    "TestCaseResultsResourceWithRawResponse",
    "AsyncTestCaseResultsResourceWithRawResponse",
    "TestCaseResultsResourceWithStreamingResponse",
    "AsyncTestCaseResultsResourceWithStreamingResponse",
    "EvaluationsResource",
    "AsyncEvaluationsResource",
    "EvaluationsResourceWithRawResponse",
    "AsyncEvaluationsResourceWithRawResponse",
    "EvaluationsResourceWithStreamingResponse",
    "AsyncEvaluationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .annotation_config_param import AnnotationConfigParam
from .multiturn_annotation_config_param import MultiturnAnnotationConfigParam
from .summarization_annotation_config_param import SummarizationAnnotationConfigParam

__all__ = [
    "EvaluationCreateParams",
    "EvaluationBuilderRequest",
    "EvaluationBuilderRequestAnnotationConfig",
    "DefaultEvaluationRequest",
    "DefaultEvaluationRequestAnnotationConfig",
]


class EvaluationBuilderRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    application_spec_id: Required[str]

    application_variant_id: Required[str]

    description: Required[str]

    evaluation_dataset_id: Required[str]

    name: Required[str]

    annotation_config: EvaluationBuilderRequestAnnotationConfig
    """Annotation configuration for tasking"""

    evaluation_config: object

    evaluation_config_id: str
    """The ID of the associated evaluation config."""

    evaluation_dataset_version: int

    question_id_to_annotation_config: Dict[str, AnnotationConfigParam]
    """Specifies the annotation configuration to use for specific questions."""

    tags: object

    type: Literal["builder"]
    """
    create standalone evaluation or build evaluation which will auto generate test
    case results
    """


EvaluationBuilderRequestAnnotationConfig: TypeAlias = Union[
    AnnotationConfigParam, MultiturnAnnotationConfigParam, SummarizationAnnotationConfigParam
]


class DefaultEvaluationRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    application_spec_id: Required[str]

    description: Required[str]

    name: Required[str]

    annotation_config: DefaultEvaluationRequestAnnotationConfig
    """Annotation configuration for tasking"""

    application_variant_id: str

    evaluation_config: object

    evaluation_config_id: str
    """The ID of the associated evaluation config."""

    question_id_to_annotation_config: Dict[str, AnnotationConfigParam]
    """Specifies the annotation configuration to use for specific questions."""

    tags: object

    type: Literal["default"]
    """
    create standalone evaluation or build evaluation which will auto generate test
    case results
    """


DefaultEvaluationRequestAnnotationConfig: TypeAlias = Union[
    AnnotationConfigParam, MultiturnAnnotationConfigParam, SummarizationAnnotationConfigParam
]

EvaluationCreateParams: TypeAlias = Union[EvaluationBuilderRequest, DefaultEvaluationRequest]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .model_deployment import ModelDeployment

__all__ = ["PaginatedModelDeployments"]


class PaginatedModelDeployments(BaseModel):
    current_page: int
    """The current page number."""

    items: List[ModelDeployment]
    """The data returned for the current page."""

    items_per_page: int
    """The number of items per page."""

    total_item_count: int
    """The total number of items of the query"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "EvaluationDatasetCreateParams",
    "ManualEvaluationDatasetRequest",
    "AutoGeneratedEvaluationDatasetRequest",
    "SafetyEvaluationDatasetRequest",
]


class ManualEvaluationDatasetRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    name: Required[str]
    """The name of the dataset"""

    schema_type: Required[Literal["GENERATION", "FLEXIBLE"]]
    """An enumeration."""

    type: Literal["manual"]
    """Manually or automatically generated dataset"""


class AutoGeneratedEvaluationDatasetRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    knowledge_base_id: Required[str]
    """ID of the knowledge base to generate the evaluation dataset from."""

    name: Required[str]
    """The name of the dataset"""

    schema_type: Required[Literal["GENERATION"]]
    """The schema type of the dataset. Only GENERATION is currently supported."""

    type: Literal["autogenerated"]
    """Manually or automatically generated dataset"""


class SafetyEvaluationDatasetRequest(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    advanced_config: Required[Dict[str, List[str]]]
    """Advanced configuration for the safety evaluation dataset."""

    harms_list: Required[List[str]]
    """List of harms to generate the dataset for."""

    name: Required[str]
    """The name of the dataset"""

    schema_type: Required[Literal["GENERATION"]]
    """The schema type of the dataset. Only GENERATION is currently supported."""

    type: Literal["safety"]
    """Manually or automatically generated dataset"""


EvaluationDatasetCreateParams: TypeAlias = Union[
    ManualEvaluationDatasetRequest, AutoGeneratedEvaluationDatasetRequest, SafetyEvaluationDatasetRequest
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import ChatCompletionsResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChatCompletions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: SGPClient) -> None:
        chat_completion = client.chat_completions.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        )
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: SGPClient) -> None:
        chat_completion = client.chat_completions.create(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            account_id="account_id",
            chat_template="chat_template",
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
            stream=False,
        )
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: SGPClient) -> None:
        response = client.chat_completions.with_raw_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_completion = response.parse()
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: SGPClient) -> None:
        with client.chat_completions.with_streaming_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_completion = response.parse()
            assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: SGPClient) -> None:
        chat_completion_stream = client.chat_completions.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        )
        chat_completion_stream.response.close()

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: SGPClient) -> None:
        chat_completion_stream = client.chat_completions.create(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            stream=True,
            account_id="account_id",
            chat_template="chat_template",
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
        )
        chat_completion_stream.response.close()

    @parametrize
    def test_raw_response_create_overload_2(self, client: SGPClient) -> None:
        response = client.chat_completions.with_raw_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_create_overload_2(self, client: SGPClient) -> None:
        with client.chat_completions.with_streaming_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True


class TestAsyncChatCompletions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        chat_completion = await async_client.chat_completions.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        )
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncSGPClient) -> None:
        chat_completion = await async_client.chat_completions.create(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            account_id="account_id",
            chat_template="chat_template",
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
            stream=False,
        )
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chat_completions.with_raw_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat_completion = await response.parse()
        assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chat_completions.with_streaming_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat_completion = await response.parse()
            assert_matches_type(ChatCompletionsResponse, chat_completion, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        chat_completion_stream = await async_client.chat_completions.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        )
        await chat_completion_stream.response.aclose()

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncSGPClient) -> None:
        chat_completion_stream = await async_client.chat_completions.create(
            messages=[
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
                {
                    "content": "string",
                    "role": "user",
                },
            ],
            model="gpt-4",
            stream=True,
            account_id="account_id",
            chat_template="chat_template",
            instructions="instructions",
            memory_strategy={
                "params": {"k": 1},
                "name": "last_k",
            },
            model_parameters={
                "frequency_penalty": 0,
                "max_tokens": 0,
                "presence_penalty": 0,
                "stop_sequences": ["string", "string", "string"],
                "temperature": 0,
                "top_k": 0,
                "top_p": 0,
            },
        )
        await chat_completion_stream.response.aclose()

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chat_completions.with_raw_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chat_completions.with_streaming_response.create(
            messages=[{"content": "string"}, {"content": "string"}, {"content": "string"}],
            model="gpt-4",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ParameterSchemaField"]


class ParameterSchemaField(BaseModel):
    description: str

    name: str

    required: bool

    type: Literal["str", "int", "float", "bool"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import RankedChunksResponse, SynthesizeChunksResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChunks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_rank(self, client: SGPClient) -> None:
        chunk = client.chunks.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        )
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    def test_method_rank_with_all_params(self, client: SGPClient) -> None:
        chunk = client.chunks.rank(
            query="query",
            rank_strategy={
                "method": "cross_encoder",
                "params": {"cross_encoder_model": "cross-encoder/ms-marco-MiniLM-L-12-v2"},
            },
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
            ],
            account_id="account_id",
            top_k=1,
        )
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    def test_raw_response_rank(self, client: SGPClient) -> None:
        response = client.chunks.with_raw_response.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = response.parse()
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    def test_streaming_response_rank(self, client: SGPClient) -> None:
        with client.chunks.with_streaming_response.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = response.parse()
            assert_matches_type(RankedChunksResponse, chunk, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_synthesis(self, client: SGPClient) -> None:
        chunk = client.chunks.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        )
        assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

    @parametrize
    def test_raw_response_synthesis(self, client: SGPClient) -> None:
        response = client.chunks.with_raw_response.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = response.parse()
        assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

    @parametrize
    def test_streaming_response_synthesis(self, client: SGPClient) -> None:
        with client.chunks.with_streaming_response.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = response.parse()
            assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncChunks:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_rank(self, async_client: AsyncSGPClient) -> None:
        chunk = await async_client.chunks.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        )
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    async def test_method_rank_with_all_params(self, async_client: AsyncSGPClient) -> None:
        chunk = await async_client.chunks.rank(
            query="query",
            rank_strategy={
                "method": "cross_encoder",
                "params": {"cross_encoder_model": "cross-encoder/ms-marco-MiniLM-L-12-v2"},
            },
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                    "attachment_url": "attachment_url",
                    "embedding": [0, 0, 0],
                    "metadata": {},
                    "title": "title",
                    "user_supplied_metadata": {},
                },
            ],
            account_id="account_id",
            top_k=1,
        )
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    async def test_raw_response_rank(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chunks.with_raw_response.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = await response.parse()
        assert_matches_type(RankedChunksResponse, chunk, path=["response"])

    @parametrize
    async def test_streaming_response_rank(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chunks.with_streaming_response.rank(
            query="query",
            rank_strategy={},
            relevant_chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = await response.parse()
            assert_matches_type(RankedChunksResponse, chunk, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_synthesis(self, async_client: AsyncSGPClient) -> None:
        chunk = await async_client.chunks.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        )
        assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

    @parametrize
    async def test_raw_response_synthesis(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.chunks.with_raw_response.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chunk = await response.parse()
        assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

    @parametrize
    async def test_streaming_response_synthesis(self, async_client: AsyncSGPClient) -> None:
        async with async_client.chunks.with_streaming_response.synthesis(
            chunks=[
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
                {
                    "chunk_id": "chunk_id",
                    "score": 0,
                    "text": "text",
                },
            ],
            query="query",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chunk = await response.parse()
            assert_matches_type(SynthesizeChunksResponse, chunk, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.evaluation_datasets import (
    EvaluationDatasetVersion,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestEvaluationDatasetVersions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        evaluation_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        evaluation_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = response.parse()
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = response.parse()
            assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.create(
                evaluation_dataset_id="",
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        evaluation_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = response.parse()
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = response.parse()
            assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
                evaluation_dataset_version_id="evaluation_dataset_version_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `evaluation_dataset_version_id` but received ''"
        ):
            client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
                evaluation_dataset_version_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        evaluation_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(SyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        evaluation_dataset_version = client.evaluation_datasets.evaluation_dataset_versions.list(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = response.parse()
        assert_matches_type(SyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = response.parse()
            assert_matches_type(
                SyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.list(
                evaluation_dataset_id="",
            )


class TestAsyncEvaluationDatasetVersions:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        evaluation_dataset_version = await async_client.evaluation_datasets.evaluation_dataset_versions.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dataset_version = await async_client.evaluation_datasets.evaluation_dataset_versions.create(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = await response.parse()
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = await response.parse()
            assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.create(
                evaluation_dataset_id="",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        evaluation_dataset_version = await async_client.evaluation_datasets.evaluation_dataset_versions.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = await response.parse()
        assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.retrieve(
            evaluation_dataset_version_id="evaluation_dataset_version_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = await response.parse()
            assert_matches_type(EvaluationDatasetVersion, evaluation_dataset_version, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
                evaluation_dataset_version_id="evaluation_dataset_version_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `evaluation_dataset_version_id` but received ''"
        ):
            await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.retrieve(
                evaluation_dataset_version_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        evaluation_dataset_version = await async_client.evaluation_datasets.evaluation_dataset_versions.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(AsyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        evaluation_dataset_version = await async_client.evaluation_datasets.evaluation_dataset_versions.list(
            evaluation_dataset_id="evaluation_dataset_id",
            account_id="account_id",
            include_archived=True,
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        evaluation_dataset_version = await response.parse()
        assert_matches_type(AsyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.evaluation_dataset_versions.with_streaming_response.list(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            evaluation_dataset_version = await response.parse()
            assert_matches_type(
                AsyncPageResponse[EvaluationDatasetVersion], evaluation_dataset_version, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.evaluation_dataset_versions.with_raw_response.list(
                evaluation_dataset_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["StringExtraInfoSchema"]


class StringExtraInfoSchema(BaseModel):
    info: str

    schema_type: Optional[Literal["STRING"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "FlexibleMessageParam",
    "EgpAPIBackendServerInternalEntitiesUserMessage",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts",
    "EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData",
    "EgpAPIBackendServerInternalEntitiesAssistantMessage",
    "EgpAPIBackendServerInternalEntitiesSystemMessage",
]


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts(
    TypedDict, total=False
):
    text: Required[str]

    type: Literal["text"]


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL(
    TypedDict, total=False
):
    url: Required[str]
    """The URL of the image. Note: only OpenAI supports this."""

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts(
    TypedDict, total=False
):
    image_url: Required[
        EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentPartsImageURL
    ]

    type: Literal["image_url"]


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData(
    TypedDict, total=False
):
    data: Required[str]
    """The base64-encoded image data."""

    media_type: Required[str]
    """The media/mime type of the image data.

    For example, 'image/png'. Check providers' documentation for supported media
    types.
    """

    detail: Literal["low", "high", "auto"]
    """Only used for OpenAI. Corresponds to OpenAI's image detail parameter."""

    type: Literal["base64"]
    """The type of the image data. Only base64 is supported."""


class EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts(
    TypedDict, total=False
):
    image_data: Required[
        EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentPartsImageData
    ]

    type: Literal["image_data"]


EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1: TypeAlias = Union[
    EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesTextUserMessageContentParts,
    EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageURLUserMessageContentParts,
    EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1EgpAPIBackendServerInternalEntitiesImageDataUserMessageContentParts,
]


class EgpAPIBackendServerInternalEntitiesUserMessage(TypedDict, total=False):
    content: Required[Union[str, Iterable[EgpAPIBackendServerInternalEntitiesUserMessageContentUnionMember1]]]

    role: Literal["user"]


class EgpAPIBackendServerInternalEntitiesAssistantMessage(TypedDict, total=False):
    content: Required[str]

    role: Literal["assistant"]


class EgpAPIBackendServerInternalEntitiesSystemMessage(TypedDict, total=False):
    content: Required[str]

    role: Literal["system"]


FlexibleMessageParam: TypeAlias = Union[
    EgpAPIBackendServerInternalEntitiesUserMessage,
    EgpAPIBackendServerInternalEntitiesAssistantMessage,
    EgpAPIBackendServerInternalEntitiesSystemMessage,
]



### application_processing_example_script.py ###
# This script demonstrates how to:
# 1. Create an application spec
# 2. Define node and edges for the application configuration
# 3. Create an application variant with the specified nodes and edges configuration
# 4. Process the application variant with an input
# This is the SDK equivalent of create an application in the Scale GenAI Web Platform:
# https://egp.dashboard.scale.com/applications

import os
import time

from scale_gp import SGPClient
from scale_gp.types.application_edge_param import ApplicationEdgeParam
from scale_gp.types.application_node_param import ApplicationNodeParam
from scale_gp.types.model_template_create_params import (
    VendorConfiguration,
    VendorConfigurationBundleConfig,
    VendorConfigurationEndpointConfig,
)
from scale_gp.types.application_configuration_param import ApplicationConfigurationParam

account_id = os.environ.get("SGP_ACCOUNT_ID", None)
api_key = os.environ.get("SGP_API_KEY", None)

assert (
    account_id is not None
), "You need to set the SGP_ACCOUNT_ID - you can find it at https://egp.dashboard.scale.com/admin/accounts"
assert api_key is not None, "You need to provide your API key - see https://egp.dashboard.scale.com/admin/api-key"

# Create an SGP Client
client = SGPClient(api_key=api_key)

### SETUP ###
# from ./ create_model_example_script.py
bundle_config = VendorConfigurationBundleConfig(image="gemini-pro", registry="aws-registry", tag="latest")
endpoint_config = VendorConfigurationEndpointConfig(
    max_workers=3,
)

vendor_configuration = VendorConfiguration(
    bundle_config=bundle_config,
    endpoint_config=endpoint_config,
)
model_template = client.model_templates.create(
    account_id=account_id,
    endpoint_type="SYNC",
    model_type="COMPLETION",
    name="Gemini-Pro Template",
    vendor_configuration=vendor_configuration,
)

model_instance = client.models.create(
    account_id=account_id,
    model_type="COMPLETION",
    name="gemini-pro",
    model_vendor="GOOGLE",
    model_template_id=model_template.id,
)
model_deployment = client.models.deployments.create(
    model_instance_id=model_instance.id, name="Gemini-Pro Deployment", account_id=account_id
)
### SETUP END ###

# 1. Create an application spec
application_spec = client.application_specs.create(
    account_id=account_id,
    description="Test application spec",
    name="test-application-spec" + str(time.time()),
)

# 2. Define edge and node parameters to be used the in the application variant graph
# for reference on how to create a model deployment, refer to ./create_model_example_script.py

input_node_id = "input_node"
completion_node_id = "completion_node"
output_node_id = "output_node"

edges = [
    ApplicationEdgeParam(
        from_node=input_node_id,
        to_node=completion_node_id,
        from_field="input-text",
        to_field="prompt",
    ),
    ApplicationEdgeParam(
        from_node=completion_node_id,
        to_node=output_node_id,
        from_field="completion",
        to_field="output-text",
    ),
]

# define the nodes. Currently each node requires the following fields:
# id: a unique identifier for the node
# application_node_schema_id: the schema id of the node. This is the type of node that is being created. Current options include:
# - text_input_schema
# - completion_model_schema
# - text_output_schema
# - knowledge_base_schema
# - reranker_schema
# - prompt_engineering_schema
# - external_endpoint_schema
# - document_input_schema
# - map_reduce_schema
# - document_search_schema
# - document_prompt_schema
# configuration: a dictionary of the configuration for the node. The keys and values of the dictionary are dependent on the schema of the node
# for example, the text_input_schema requires a "input-text" field, which is a dictionary with a "type" TEXT and "value" input type fields

# The following nodes are defined in this example:
# 1. input_node: a text input node with the input field "input-text". Each app needs at least one input node which is the entrypoint to the app
# 2. completion_node: a completion model node with the configuration field "completion_model_id" which is the id of the deployed completion model we want to use inside the application
# 3. output_node: a text output node, which defines what processing the application will return. Each app needs at least one output node which is the exitpoint of the app

# overall, we are creating a very basic completion application that takes in a text input, processes it with a completion model, and returns the completion result
nodes = [
    ApplicationNodeParam(
        id=input_node_id,
        application_node_schema_id="text_input_schema",
        configuration={"input-text": {"type": "TEXT", "value": "Completion prompt"}},
    ),
    ApplicationNodeParam(
        id=completion_node_id,
        application_node_schema_id="completion_model_schema",
        configuration={
            "completion_model_id": {
                "type": "TEXT",
                "value": model_deployment.id,  # define the deployed completion model we want to use inside the application
            }
        },
    ),
    ApplicationNodeParam(
        id=output_node_id,
        application_node_schema_id="text_output_schema",
        configuration={"completion": {"type": "TEXT", "value": "completion result"}},
    ),
]
configuration = ApplicationConfigurationParam(edges=edges, nodes=nodes)

# 3. create an application variant with the specified nodes and edges configuration
application_variant = client.application_variants.create(
    account_id=account_id,
    application_spec_id=application_spec.id,
    name="test-application-variant",
    version="V0",
    description="Test application variant",
    configuration=configuration,
)

# 4. To interact with the variant, we need to provide an input for each input node in the graph configuration.
# Input nodes are an node with the application_node_schema_id of text_input_schema
# In this case, the input node has id "input_node" and the input field is "input-text"
variant_response = client.application_variants.process(
    application_variant_id=application_variant.id,
    inputs={"input_node": {"input-text": "Why is the sky blue?"}},
)

print(variant_response["output-text"])
# Returns:
#
# The sky appears blue because of a phenomenon called Rayleigh scattering. Sunlight is made up of all the colors of the rainbow, each with a different wavelength. When sunlight enters the Earth's atmosphere, it interacts with the molecules of air. Blue light has a shorter wavelength than other colors, so it is scattered more easily by the air molecules. This scattered blue light is what we see as the blue sky.

# Here are some additional details about Rayleigh scattering:

# * It is named after Lord Rayleigh, who first explained it in the 19th century.
# * It is most effective at scattering shorter wavelengths of light, such as blue and violet.
# * It is the reason why the sky is blue during the day and why the sun appears yellow or orange at sunrise and sunset.
# * It also explains why the sky is red or orange during periods of heavy air pollution, such as smog.

# I hope this explanation is helpful. Please let me know if you have any other questions.



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["EvaluationDatasetPublishParams"]


class EvaluationDatasetPublishParams(TypedDict, total=False):
    force: bool
    """Force approve an evaluation dataset"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..types import agent_execute_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.execute_agent_response import ExecuteAgentResponse

__all__ = ["AgentsResource", "AsyncAgentsResource"]


class AgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AgentsResourceWithStreamingResponse(self)

    def execute(
        self,
        *,
        messages: Iterable[agent_execute_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "claude-instant-1",
            "claude-instant-1.1",
            "claude-2",
            "claude-2.0",
        ],
        tools: Iterable[agent_execute_params.Tool],
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: agent_execute_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: agent_execute_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExecuteAgentResponse:
        """### Description

        Executes one Agent inference step.

        Given a list of messages and a list of tools
        to ask for help from, the Agent will either respond with a final answer directly
        or ask the user to execute a tool to provide more information.

        ### Details

        An Agent is a component that utilizes a Language Model (LLM) as an interpreter
        and decision maker. Unlike asking an LLM for a direct response, communicating
        with an agent consists of a running dialogue where an agent can optionally ask
        the user to execute specialized tools for specific tasks, such as calculations,
        web searches, or accessing custom data from private knowledge bases.

        An agent is designed to be stateless, emitting outputs one step at a time. This
        means that client-side applications are responsible for managing message
        history, tool execution, and responses. This grants users greater flexibility to
        write and execute custom tools and maintain explicit control over their message
        history.

        #### Message Types

        - `User Message`: A message from the user to the agent.
        - `System Message`: An informational text message from the system to guide the
          agent. It is not a user message or agent message because it did not come from
          either entity.
        - `Agent Message`: A message from the agent to the client. It will contain
          either a final answer as `content` or a request for the user to execute a tool
          as a `tool_request`.
        - `Tool Message`: A message from the user to the agent that contains the output
          of a tool execution. The tool message will be processed by the agent and the
          agent will respond with either a final answer or another tool request.

        #### Agent Instructions

        Instructions are used to guide the agent's decision making process and output
        generation.

        Good prompt engineering is crucial to getting performant results from the agent.
        If you are having trouble getting the agent to perform well, try writing more
        specific instructions before trying more expensive techniques such as swapping
        in other models or finetuning the underlying LLM.

        For example, the default instructions we set for the agent are the following:

        > You are an AI assistant that helps users with their questions. You can answer
        > questions directly or acquire information from any of the attached tools to
        > assist you. Always answer the user's most recent query to the best of your
        > knowledge.

        > When asked about what tools are available, you must list each attached tool's
        > name and description. When asked about what you can do, mention that in
        > addition to your normal capabilities, you can also use the attached tools by
        > listing their names and descriptions. You cannot use any other tools other
        > than the ones provided to you explicitly.

        ### Restrictions and Limits

        **Message Limits**:

        - The message list is not limited by length, but by the context limit of the
          underlying language model. If you are getting an error regarding the
          underlying model's context limit, try using a memory strategy to condense the
          input messages.

        **Model Restrictions**:

        - Currently, only closed source models like GPT and Claude are supported due to
          the limitations of open source models when it comes to tool selection,
          generating tool arguments in valid JSON, and planning out multi-step tool
          execution. Specialized fine-tuning will likely be required to make open source
          models compatible with agents.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for the agent. We only support the models listed here
              so far.

          tools: The list of specs of tools that the agent can use. Each spec must contain a
              `name` key set to the name of the tool, a `description` key set to the
              description of the tool, and an `arguments` key set to a JSON Schema compliant
              object describing the tool arguments.

              The name and description of each tool is used by the agent to decide when to use
              certain tools. Because some queries are complex and may require multiple tools
              to complete, it is important to make these descriptions as informative as
              possible. If a tool is not being chosen when it should, it is common practice to
              tune the description of the tool to make it more apparent to the agent when the
              tool can be used effectively.

          instructions: The initial instructions to provide to the agent.

              Use this to guide the agent to act in more specific ways. For example, if you
              have specific rules you want to restrict the agent to follow you can specify
              them here. For example, if I want the agent to always use certain tools before
              others, I can write that rule in these instructions.

              Good prompt engineering is crucial to getting performant results from the agent.
              If you are having trouble getting the agent to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the agent model, such as temperature, max_tokens,
              and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v4/agents/execute",
            body=maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "tools": tools,
                    "instructions": instructions,
                    "memory_strategy": memory_strategy,
                    "model_parameters": model_parameters,
                },
                agent_execute_params.AgentExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExecuteAgentResponse,
        )


class AsyncAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return the
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/scaleapi/sgp-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/scaleapi/sgp-python#with_streaming_response
        """
        return AsyncAgentsResourceWithStreamingResponse(self)

    async def execute(
        self,
        *,
        messages: Iterable[agent_execute_params.Message],
        model: Literal[
            "gpt-4",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-16k-0613",
            "claude-instant-1",
            "claude-instant-1.1",
            "claude-2",
            "claude-2.0",
        ],
        tools: Iterable[agent_execute_params.Tool],
        instructions: str | NotGiven = NOT_GIVEN,
        memory_strategy: agent_execute_params.MemoryStrategy | NotGiven = NOT_GIVEN,
        model_parameters: agent_execute_params.ModelParameters | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExecuteAgentResponse:
        """### Description

        Executes one Agent inference step.

        Given a list of messages and a list of tools
        to ask for help from, the Agent will either respond with a final answer directly
        or ask the user to execute a tool to provide more information.

        ### Details

        An Agent is a component that utilizes a Language Model (LLM) as an interpreter
        and decision maker. Unlike asking an LLM for a direct response, communicating
        with an agent consists of a running dialogue where an agent can optionally ask
        the user to execute specialized tools for specific tasks, such as calculations,
        web searches, or accessing custom data from private knowledge bases.

        An agent is designed to be stateless, emitting outputs one step at a time. This
        means that client-side applications are responsible for managing message
        history, tool execution, and responses. This grants users greater flexibility to
        write and execute custom tools and maintain explicit control over their message
        history.

        #### Message Types

        - `User Message`: A message from the user to the agent.
        - `System Message`: An informational text message from the system to guide the
          agent. It is not a user message or agent message because it did not come from
          either entity.
        - `Agent Message`: A message from the agent to the client. It will contain
          either a final answer as `content` or a request for the user to execute a tool
          as a `tool_request`.
        - `Tool Message`: A message from the user to the agent that contains the output
          of a tool execution. The tool message will be processed by the agent and the
          agent will respond with either a final answer or another tool request.

        #### Agent Instructions

        Instructions are used to guide the agent's decision making process and output
        generation.

        Good prompt engineering is crucial to getting performant results from the agent.
        If you are having trouble getting the agent to perform well, try writing more
        specific instructions before trying more expensive techniques such as swapping
        in other models or finetuning the underlying LLM.

        For example, the default instructions we set for the agent are the following:

        > You are an AI assistant that helps users with their questions. You can answer
        > questions directly or acquire information from any of the attached tools to
        > assist you. Always answer the user's most recent query to the best of your
        > knowledge.

        > When asked about what tools are available, you must list each attached tool's
        > name and description. When asked about what you can do, mention that in
        > addition to your normal capabilities, you can also use the attached tools by
        > listing their names and descriptions. You cannot use any other tools other
        > than the ones provided to you explicitly.

        ### Restrictions and Limits

        **Message Limits**:

        - The message list is not limited by length, but by the context limit of the
          underlying language model. If you are getting an error regarding the
          underlying model's context limit, try using a memory strategy to condense the
          input messages.

        **Model Restrictions**:

        - Currently, only closed source models like GPT and Claude are supported due to
          the limitations of open source models when it comes to tool selection,
          generating tool arguments in valid JSON, and planning out multi-step tool
          execution. Specialized fine-tuning will likely be required to make open source
          models compatible with agents.

        Args:
          messages: The list of messages in the conversation.

              Expand each message type to see how it works and when to use it. Most
              conversations should begin with a single `user` message.

          model: The ID of the model to use for the agent. We only support the models listed here
              so far.

          tools: The list of specs of tools that the agent can use. Each spec must contain a
              `name` key set to the name of the tool, a `description` key set to the
              description of the tool, and an `arguments` key set to a JSON Schema compliant
              object describing the tool arguments.

              The name and description of each tool is used by the agent to decide when to use
              certain tools. Because some queries are complex and may require multiple tools
              to complete, it is important to make these descriptions as informative as
              possible. If a tool is not being chosen when it should, it is common practice to
              tune the description of the tool to make it more apparent to the agent when the
              tool can be used effectively.

          instructions: The initial instructions to provide to the agent.

              Use this to guide the agent to act in more specific ways. For example, if you
              have specific rules you want to restrict the agent to follow you can specify
              them here. For example, if I want the agent to always use certain tools before
              others, I can write that rule in these instructions.

              Good prompt engineering is crucial to getting performant results from the agent.
              If you are having trouble getting the agent to perform well, try writing more
              specific instructions here before trying more expensive techniques such as
              swapping in other models or finetuning the underlying LLM.

          memory_strategy: The memory strategy to use for the agent. A memory strategy is a way to prevent
              the underlying LLM's context limit from being exceeded. Each memory strategy
              uses a different technique to condense the input message list into a smaller
              payload for the underlying LLM.

              We only support the Last K memory strategy right now, but will be adding new
              strategies soon.

          model_parameters: Configuration parameters for the agent model, such as temperature, max_tokens,
              and stop_sequences.

              If not specified, the default value are:

              - temperature: 0.2
              - max_tokens: None (limited by the model's max tokens)
              - stop_sequences: None

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v4/agents/execute",
            body=await async_maybe_transform(
                {
                    "messages": messages,
                    "model": model,
                    "tools": tools,
                    "instructions": instructions,
                    "memory_strategy": memory_strategy,
                    "model_parameters": model_parameters,
                },
                agent_execute_params.AgentExecuteParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ExecuteAgentResponse,
        )


class AgentsResourceWithRawResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.execute = to_raw_response_wrapper(
            agents.execute,
        )


class AsyncAgentsResourceWithRawResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.execute = async_to_raw_response_wrapper(
            agents.execute,
        )


class AgentsResourceWithStreamingResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.execute = to_streamed_response_wrapper(
            agents.execute,
        )


class AsyncAgentsResourceWithStreamingResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.execute = async_to_streamed_response_wrapper(
            agents.execute,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ApplicationSpecCreateParams"]


class ApplicationSpecCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The ID of the account that owns the given entity."""

    description: Required[str]
    """The description of the Application Spec"""

    name: Required[str]
    """The name of the Application Spec"""

    run_online_evaluation: bool
    """Whether the application spec should run online evaluation, default is `false`"""

    theme_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from ..._models import BaseModel

__all__ = ["GenericDeleteResponse"]


class GenericDeleteResponse(BaseModel):
    count: int

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.pagination import SyncGenerationJobsPagination, AsyncGenerationJobsPagination
from scale_gp.types.evaluation_datasets import (
    EvaluationDatasetGenerationJob,
    EvaluationDatasetGenerationJobResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGenerationJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        generation_job = client.evaluation_datasets.generation_jobs.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        generation_job = client.evaluation_datasets.generation_jobs.create(
            evaluation_dataset_id="evaluation_dataset_id",
            advanced_config={"foo": ["string", "string", "string"]},
            custom_instructions="custom_instructions",
            group_by_artifact_id=True,
            harms_list=["string", "string", "string"],
            num_test_cases=0,
        )
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.generation_jobs.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = response.parse()
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.evaluation_datasets.generation_jobs.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = response.parse()
            assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.create(
                evaluation_dataset_id="",
            )

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        generation_job = client.evaluation_datasets.generation_jobs.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = response.parse()
        assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.evaluation_datasets.generation_jobs.with_streaming_response.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = response.parse()
            assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
                generation_job_id="generation_job_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `generation_job_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
                generation_job_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        generation_job = client.evaluation_datasets.generation_jobs.list(
            "evaluation_dataset_id",
        )
        assert_matches_type(
            SyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
        )

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.generation_jobs.with_raw_response.list(
            "evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = response.parse()
        assert_matches_type(
            SyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
        )

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.evaluation_datasets.generation_jobs.with_streaming_response.list(
            "evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = response.parse()
            assert_matches_type(
                SyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.list(
                "",
            )

    @parametrize
    def test_method_cancel(self, client: SGPClient) -> None:
        generation_job = client.evaluation_datasets.generation_jobs.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(object, generation_job, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: SGPClient) -> None:
        response = client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = response.parse()
        assert_matches_type(object, generation_job, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: SGPClient) -> None:
        with client.evaluation_datasets.generation_jobs.with_streaming_response.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = response.parse()
            assert_matches_type(object, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
                generation_job_id="generation_job_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `generation_job_id` but received ''"):
            client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
                generation_job_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )


class TestAsyncGenerationJobs:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        generation_job = await async_client.evaluation_datasets.generation_jobs.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        generation_job = await async_client.evaluation_datasets.generation_jobs.create(
            evaluation_dataset_id="evaluation_dataset_id",
            advanced_config={"foo": ["string", "string", "string"]},
            custom_instructions="custom_instructions",
            group_by_artifact_id=True,
            harms_list=["string", "string", "string"],
            num_test_cases=0,
        )
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.generation_jobs.with_raw_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = await response.parse()
        assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.generation_jobs.with_streaming_response.create(
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = await response.parse()
            assert_matches_type(EvaluationDatasetGenerationJobResponse, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.create(
                evaluation_dataset_id="",
            )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        generation_job = await async_client.evaluation_datasets.generation_jobs.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = await response.parse()
        assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.generation_jobs.with_streaming_response.retrieve(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = await response.parse()
            assert_matches_type(EvaluationDatasetGenerationJob, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
                generation_job_id="generation_job_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `generation_job_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.retrieve(
                generation_job_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        generation_job = await async_client.evaluation_datasets.generation_jobs.list(
            "evaluation_dataset_id",
        )
        assert_matches_type(
            AsyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
        )

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.generation_jobs.with_raw_response.list(
            "evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = await response.parse()
        assert_matches_type(
            AsyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
        )

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.generation_jobs.with_streaming_response.list(
            "evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = await response.parse()
            assert_matches_type(
                AsyncGenerationJobsPagination[EvaluationDatasetGenerationJob], generation_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.list(
                "",
            )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncSGPClient) -> None:
        generation_job = await async_client.evaluation_datasets.generation_jobs.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )
        assert_matches_type(object, generation_job, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation_job = await response.parse()
        assert_matches_type(object, generation_job, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncSGPClient) -> None:
        async with async_client.evaluation_datasets.generation_jobs.with_streaming_response.cancel(
            generation_job_id="generation_job_id",
            evaluation_dataset_id="evaluation_dataset_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation_job = await response.parse()
            assert_matches_type(object, generation_job, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `evaluation_dataset_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
                generation_job_id="generation_job_id",
                evaluation_dataset_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `generation_job_id` but received ''"):
            await async_client.evaluation_datasets.generation_jobs.with_raw_response.cancel(
                generation_job_id="",
                evaluation_dataset_id="evaluation_dataset_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "DeploymentUpdateParams",
    "VendorConfiguration",
    "VendorConfigurationLaunchDeploymentVendorConfiguration",
    "VendorConfigurationLlmEngineDeploymentVendorConfiguration",
]


class DeploymentUpdateParams(TypedDict, total=False):
    model_instance_id: Required[str]

    deployment_metadata: object

    model_creation_parameters: object

    name: str

    vendor_configuration: VendorConfiguration


class VendorConfigurationLaunchDeploymentVendorConfiguration(TypedDict, total=False):
    max_workers: int

    min_workers: int

    per_worker: int
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    vendor: Literal["LAUNCH"]


class VendorConfigurationLlmEngineDeploymentVendorConfiguration(TypedDict, total=False):
    base_model_name: str

    checkpoint_path: str

    cpus: int

    gpu_type: Literal[
        "nvidia-tesla-t4",
        "nvidia-ampere-a10",
        "nvidia-ampere-a100",
        "nvidia-ampere-a100e",
        "nvidia-hopper-h100",
        "nvidia-hopper-h100-1g20gb",
        "nvidia-hopper-h100-3g40gb",
    ]
    """An enumeration."""

    gpus: int

    high_priority: bool

    inference_framework_image_tag: str

    max_workers: int

    memory: str

    min_workers: int

    model_name: str

    num_shards: int

    per_worker: int
    """The maximum number of concurrent requests that an individual worker can service.

    Launch automatically scales the number of workers for the endpoint so that each
    worker is processing `per_worker` requests, subject to the limits defined by
    `min_workers` and `max_workers`.

    - If the average number of concurrent requests per worker is lower than
      `per_worker`, then the number of workers will be reduced. - Otherwise, if the
      average number of concurrent requests per worker is higher than `per_worker`,
      then the number of workers will be increased to meet the elevated traffic.

    Here is our recommendation for computing `per_worker`:

    1. Compute `min_workers` and `max_workers` per your minimum and maximum
       throughput requirements. 2. Determine a value for the maximum number of
       concurrent requests in the workload. Divide this number by `max_workers`.
       Doing this ensures that the number of workers will "climb" to `max_workers`.
    """

    storage: str

    vendor: Literal["LLMENGINE"]


VendorConfiguration: TypeAlias = Union[
    VendorConfigurationLaunchDeploymentVendorConfiguration, VendorConfigurationLlmEngineDeploymentVendorConfiguration
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import ModelTemplate
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestModelTemplates:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        model_template = client.model_templates.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        model_template = client.model_templates.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                    "command": ["string", "string", "string"],
                    "env": {"foo": "string"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_initial_delay_seconds": 0,
                    "streaming_command": ["string", "string", "string"],
                    "streaming_predict_route": "streaming_predict_route",
                },
                "endpoint_config": {
                    "cpus": 0,
                    "endpoint_type": "SYNC",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "storage": "storage",
                },
                "fine_tuning_job_bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                    "command": ["string", "string", "string"],
                    "env": {"foo": "string"},
                    "mount_location": "mount_location",
                    "resources": {
                        "cpus": 0,
                        "gpu_type": "nvidia-tesla-t4",
                        "gpus": 0,
                        "memory": "memory",
                        "storage": "storage",
                    },
                    "training_dataset_schema_type": "GENERATION",
                },
                "vendor": "LAUNCH",
            },
            endpoint_protocol="SGP",
            model_creation_parameters_schema={
                "parameters": [
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                ]
            },
            model_request_parameters_schema={
                "parameters": [
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                ]
            },
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.model_templates.with_raw_response.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = response.parse()
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.model_templates.with_streaming_response.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = response.parse()
            assert_matches_type(ModelTemplate, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        model_template = client.model_templates.retrieve(
            "model_template_id",
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.model_templates.with_raw_response.retrieve(
            "model_template_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = response.parse()
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.model_templates.with_streaming_response.retrieve(
            "model_template_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = response.parse()
            assert_matches_type(ModelTemplate, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_template_id` but received ''"):
            client.model_templates.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        model_template = client.model_templates.list()
        assert_matches_type(SyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        model_template = client.model_templates.list(
            account_id="account_id",
            limit=1,
            page=1,
        )
        assert_matches_type(SyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.model_templates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = response.parse()
        assert_matches_type(SyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.model_templates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = response.parse()
            assert_matches_type(SyncPageResponse[ModelTemplate], model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        model_template = client.model_templates.delete(
            "model_template_id",
        )
        assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.model_templates.with_raw_response.delete(
            "model_template_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = response.parse()
        assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.model_templates.with_streaming_response.delete(
            "model_template_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = response.parse()
            assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_template_id` but received ''"):
            client.model_templates.with_raw_response.delete(
                "",
            )


class TestAsyncModelTemplates:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                    "command": ["string", "string", "string"],
                    "env": {"foo": "string"},
                    "healthcheck_route": "healthcheck_route",
                    "predict_route": "predict_route",
                    "readiness_initial_delay_seconds": 0,
                    "streaming_command": ["string", "string", "string"],
                    "streaming_predict_route": "streaming_predict_route",
                },
                "endpoint_config": {
                    "cpus": 0,
                    "endpoint_type": "SYNC",
                    "gpu_type": "nvidia-tesla-t4",
                    "gpus": 0,
                    "high_priority": True,
                    "max_workers": 0,
                    "memory": "memory",
                    "min_workers": 0,
                    "per_worker": 0,
                    "storage": "storage",
                },
                "fine_tuning_job_bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                    "command": ["string", "string", "string"],
                    "env": {"foo": "string"},
                    "mount_location": "mount_location",
                    "resources": {
                        "cpus": 0,
                        "gpu_type": "nvidia-tesla-t4",
                        "gpus": 0,
                        "memory": "memory",
                        "storage": "storage",
                    },
                    "training_dataset_schema_type": "GENERATION",
                },
                "vendor": "LAUNCH",
            },
            endpoint_protocol="SGP",
            model_creation_parameters_schema={
                "parameters": [
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                ]
            },
            model_request_parameters_schema={
                "parameters": [
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                    {
                        "description": "description",
                        "name": "name",
                        "required": True,
                        "type": "str",
                    },
                ]
            },
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.model_templates.with_raw_response.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = await response.parse()
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.model_templates.with_streaming_response.create(
            account_id="account_id",
            endpoint_type="SYNC",
            model_type="COMPLETION",
            name="name",
            vendor_configuration={
                "bundle_config": {
                    "image": "image",
                    "registry": "registry",
                    "tag": "tag",
                }
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = await response.parse()
            assert_matches_type(ModelTemplate, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.retrieve(
            "model_template_id",
        )
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.model_templates.with_raw_response.retrieve(
            "model_template_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = await response.parse()
        assert_matches_type(ModelTemplate, model_template, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.model_templates.with_streaming_response.retrieve(
            "model_template_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = await response.parse()
            assert_matches_type(ModelTemplate, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_template_id` but received ''"):
            await async_client.model_templates.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.list()
        assert_matches_type(AsyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.list(
            account_id="account_id",
            limit=1,
            page=1,
        )
        assert_matches_type(AsyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.model_templates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = await response.parse()
        assert_matches_type(AsyncPageResponse[ModelTemplate], model_template, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.model_templates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = await response.parse()
            assert_matches_type(AsyncPageResponse[ModelTemplate], model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        model_template = await async_client.model_templates.delete(
            "model_template_id",
        )
        assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.model_templates.with_raw_response.delete(
            "model_template_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model_template = await response.parse()
        assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.model_templates.with_streaming_response.delete(
            "model_template_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model_template = await response.parse()
            assert_matches_type(GenericDeleteResponse, model_template, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_template_id` but received ''"):
            await async_client.model_templates.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict
from typing_extensions import TypeAlias

from .application_node_schema_registry_record import ApplicationNodeSchemaRegistryRecord

__all__ = ["ApplicationSchemaRetrieveResponse"]

ApplicationSchemaRetrieveResponse: TypeAlias = Dict[str, ApplicationNodeSchemaRegistryRecord]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TestCaseResultCreateParams"]


class TestCaseResultCreateParams(TypedDict, total=False):
    application_spec_id: Required[str]

    evaluation_dataset_version_num: Required[str]

    test_case_evaluation_data: Required[object]

    test_case_id: Required[str]

    account_id: str
    """The ID of the account that owns the given entity."""

    annotated_by_user_id: str
    """The user who annotated the task."""

    audit_comment: str

    audit_required: bool

    audit_status: Literal["UNAUDITED", "FIXED", "APPROVED"]
    """An enumeration."""

    result: object

    time_spent_labeling_s: int
    """The time spent labeling in seconds."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from scale_gp import SGPClient, AsyncSGPClient
from tests.utils import assert_matches_type
from scale_gp.types import (
    ModelInstance,
    ModelInstanceWithViews,
)
from scale_gp.pagination import SyncPageResponse, AsyncPageResponse
from scale_gp.types.shared import GenericDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestModels:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: SGPClient) -> None:
        model = client.models.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: SGPClient) -> None:
        model = client.models.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
            base_model_id="base_model_id",
            base_model_metadata={
                "delivery_date": "delivery_date",
                "model_developer": "model_developer",
                "model_license_url": "model_license_url",
                "model_details": {
                    "alignments": 0,
                    "languages": 0,
                    "number_of_parameters": 0,
                    "token_context_window": 0,
                },
                "ui_model_section_type": "PARTNER",
            },
            description="description",
            model_card="model_card",
            model_creation_parameters={},
            model_group_id="model_group_id",
            model_template_id="model_template_id",
            model_vendor="OPENAI",
            training_data_card="training_data_card",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelInstance, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: SGPClient) -> None:
        model = client.models.retrieve(
            model_id="model_id",
        )
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: SGPClient) -> None:
        model = client.models.retrieve(
            model_id="model_id",
            view=["Deployments", "ModelGroup"],
        )
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.retrieve(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.retrieve(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelInstanceWithViews, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.retrieve(
                model_id="",
            )

    @parametrize
    def test_method_update(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: SGPClient) -> None:
        model = client.models.update(
            model_id="model_id",
            base_model_id="base_model_id",
            base_model_metadata={
                "delivery_date": "delivery_date",
                "model_developer": "model_developer",
                "model_license_url": "model_license_url",
                "model_details": {
                    "alignments": 0,
                    "languages": 0,
                    "number_of_parameters": 0,
                    "token_context_window": 0,
                },
                "ui_model_section_type": "PARTNER",
            },
            description="description",
            model_card="model_card",
            model_creation_parameters={},
            model_group_id="model_group_id",
            model_template_id="model_template_id",
            model_type="COMPLETION",
            model_vendor="OPENAI",
            name="name",
            training_data_card="training_data_card",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.update(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.update(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelInstance, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.update(
                model_id="",
            )

    @parametrize
    def test_method_list(self, client: SGPClient) -> None:
        model = client.models.list()
        assert_matches_type(SyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: SGPClient) -> None:
        model = client.models.list(
            account_id="account_id",
            limit=1,
            model_group_id=0,
            model_type=0,
            page=1,
            sort_by=["model_vendor:asc", "model_vendor:desc", "base_model_id:asc"],
            view=["Deployments", "ModelGroup"],
        )
        assert_matches_type(SyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(SyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(SyncPageResponse[ModelInstanceWithViews], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: SGPClient) -> None:
        model = client.models.delete(
            "model_id",
        )
        assert_matches_type(GenericDeleteResponse, model, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: SGPClient) -> None:
        response = client.models.with_raw_response.delete(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(GenericDeleteResponse, model, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: SGPClient) -> None:
        with client.models.with_streaming_response.delete(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(GenericDeleteResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: SGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            client.models.with_raw_response.delete(
                "",
            )


class TestAsyncModels:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
            base_model_id="base_model_id",
            base_model_metadata={
                "delivery_date": "delivery_date",
                "model_developer": "model_developer",
                "model_license_url": "model_license_url",
                "model_details": {
                    "alignments": 0,
                    "languages": 0,
                    "number_of_parameters": 0,
                    "token_context_window": 0,
                },
                "ui_model_section_type": "PARTNER",
            },
            description="description",
            model_card="model_card",
            model_creation_parameters={},
            model_group_id="model_group_id",
            model_template_id="model_template_id",
            model_vendor="OPENAI",
            training_data_card="training_data_card",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.create(
            account_id="account_id",
            model_type="COMPLETION",
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelInstance, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.retrieve(
            model_id="model_id",
        )
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.retrieve(
            model_id="model_id",
            view=["Deployments", "ModelGroup"],
        )
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.retrieve(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelInstanceWithViews, model, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.retrieve(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelInstanceWithViews, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.retrieve(
                model_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.update(
            model_id="model_id",
            base_model_id="base_model_id",
            base_model_metadata={
                "delivery_date": "delivery_date",
                "model_developer": "model_developer",
                "model_license_url": "model_license_url",
                "model_details": {
                    "alignments": 0,
                    "languages": 0,
                    "number_of_parameters": 0,
                    "token_context_window": 0,
                },
                "ui_model_section_type": "PARTNER",
            },
            description="description",
            model_card="model_card",
            model_creation_parameters={},
            model_group_id="model_group_id",
            model_template_id="model_template_id",
            model_type="COMPLETION",
            model_vendor="OPENAI",
            name="name",
            training_data_card="training_data_card",
        )
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.update(
            model_id="model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelInstance, model, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.update(
            model_id="model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelInstance, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.update(
                model_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.list()
        assert_matches_type(AsyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.list(
            account_id="account_id",
            limit=1,
            model_group_id=0,
            model_type=0,
            page=1,
            sort_by=["model_vendor:asc", "model_vendor:desc", "base_model_id:asc"],
            view=["Deployments", "ModelGroup"],
        )
        assert_matches_type(AsyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(AsyncPageResponse[ModelInstanceWithViews], model, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(AsyncPageResponse[ModelInstanceWithViews], model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncSGPClient) -> None:
        model = await async_client.models.delete(
            "model_id",
        )
        assert_matches_type(GenericDeleteResponse, model, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncSGPClient) -> None:
        response = await async_client.models.with_raw_response.delete(
            "model_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(GenericDeleteResponse, model, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncSGPClient) -> None:
        async with async_client.models.with_streaming_response.delete(
            "model_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(GenericDeleteResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncSGPClient) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `model_id` but received ''"):
            await async_client.models.with_raw_response.delete(
                "",
            )



from __future__ import annotations

import os
from typing import Any, Dict

import pytest

from scale_gp import SGPClient
from tests.utils import assert_matches_type
from scale_gp.lib.external_applications import ExternalApplication, ExternalApplicationOutputCompletion
from scale_gp.types.evaluation_datasets.test_case import TestCase

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExternalApplicationsLibrary:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_initialize(self, client: SGPClient) -> None:
        ExternalApplication(client).initialize(
            application_variant_id="application_variant_id",
            application=lambda prompt: ExternalApplicationOutputCompletion(generation_output=prompt),
        )

    @parametrize
    def test_generate_application_with_prompt(self, client: SGPClient) -> None:
        def application(prompt: str) -> ExternalApplicationOutputCompletion:
            assert_matches_type(str, prompt, path=["application", "prompt"])
            return ExternalApplicationOutputCompletion(generation_output=prompt)

        external_application = ExternalApplication(client).initialize(
            application_variant_id="application_variant_id",
            application=application,
        )
        external_application.generate_outputs(
            evaluation_dataset_id="evaluation_dataset_id",
            evaluation_dataset_version=int(),
        )

    @parametrize
    def test_generate_application_with_prompt_and_test_case(self, client: SGPClient) -> None:
        def application(prompt: str, test_case: TestCase) -> ExternalApplicationOutputCompletion:
            assert_matches_type(str, prompt, path=["application", "prompt"])
            assert_matches_type(TestCase, test_case, path=["application", "test_case"])
            return ExternalApplicationOutputCompletion(generation_output=prompt)

        external_application = ExternalApplication(client).initialize(
            application_variant_id="application_variant_id",
            application=application,
        )
        external_application.generate_outputs(
            evaluation_dataset_id="evaluation_dataset_id",
            evaluation_dataset_version=int(),
        )

    @parametrize
    def test_generate_application_with_flexible_prompt_and_test_case(self, client: SGPClient) -> None:
        def application(prompt: Dict[str, Any], test_case: TestCase) -> ExternalApplicationOutputCompletion:
            assert_matches_type(Dict[str, Any], prompt, path=["application", "prompt"])
            assert_matches_type(TestCase, test_case, path=["application", "test_case"])
            return ExternalApplicationOutputCompletion(generation_output="output")

        external_application = ExternalApplication(client).initialize(
            application_variant_id="application_variant_id",
            application=application,
        )
        external_application.generate_outputs(
            evaluation_dataset_id="evaluation_dataset_id",
            evaluation_dataset_version=int(),
        )
